package esami

import (
	"bufio"
	"errors"
	"fmt"
	"os"
	"regexp"
	"strconv"
)

/*
	Formato di salvataggio:
		- file di testo
		- una riga per ogni studente
		- sulla riga compare prima il nome e poi l'elenco dei suoi voti; il nome e i voti sono separati da TAB
		- ci possono essere molti TAB consecutivi e questo non deve far esplodere il programma
*/

func ScriviFile(m Nomi2Voti, nomeFile string) error {
	f, err := os.Create(nomeFile)
	defer f.Close()
	if err != nil {
		return err
	}
	for nome, voti := range m {
		fmt.Fprintf(f, "%s", nome)
		for _, v := range voti {
			fmt.Fprintf(f, "\t%d", v)
		}
		fmt.Fprintln(f)
	}
	return nil
}

func parseLine(line string) (nome string, voti Voti, err error) {
	regexpNome := regexp.MustCompile("[a-zA-Z][a-zA-Z' -]+") // Espressione regolare per un nome
	regexpVoti := regexp.MustCompile("[0-9]+")               // Espressione regolare per i voti
	s := regexpNome.FindAllString(line, -1)
	if len(s) != 1 {
		return "", nil, errors.New("Sulla riga " + line + " ci sono zero o più di un nome!")
	}
	nome = s[0] // La slice contiene un solo elemento, il nome dello studente su quella riga

	v := regexpVoti.FindAllString(line, -1)
	for _, x := range v {
		n, err := strconv.Atoi(x)
		if err != nil {
			return "", nil, err
		}
		voti = append(voti, n)
	}
	return
}

func LeggiFile(nomeFile string) (Nomi2Voti, error) {
	f, err := os.Open(nomeFile)
	defer f.Close()
	if err != nil {
		return nil, err
	}
	res := make(Nomi2Voti)


	scanner := bufio.NewScanner(f)

	for scanner.Scan() {
		line := scanner.Text()
		nome, voti, err := parseLine(line)
		if err != nil {
			return nil, err
		}
		res[nome] = voti

	}
	return res, nil

}
